@@ -1,17 +1,31 @@
 /**********************************************************************
  *
  * PostGIS - Spatial Types for PostgreSQL
+ * http://postgis.net
  *
- * Copyright 2011 Sandro Santilli <strk@keybit.net>
+ * PostGIS is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * PostGIS is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with PostGIS.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ **********************************************************************
+ *
+ * Copyright 2011 Sandro Santilli <strk@kbt.io>
  * Copyright 2011 Paul Ramsey <pramsey@cleverelephant.ca>
  * Copyright 2007-2008 Mark Cave-Ayland
  * Copyright 2001-2006 Refractions Research Inc.
  *
- * This is free software; you can redistribute and/or modify it under
- * the terms of the GNU General Public Licence. See the COPYING file.
- *
  **********************************************************************/
 
+
 #ifndef _LIBLWGEOM_H
 #define _LIBLWGEOM_H 1
 
@@ -40,17 +54,17 @@
 * ../loader/shp2pgsql.c are examples of non-PostGIS applications using liblwgeom.
 *
 * Programs using this library can install their custom memory managers and error
-* handlers by calling the lwgeom_set_handlers() function, otherwise the default 
+* handlers by calling the lwgeom_set_handlers() function, otherwise the default
 * ones will be used.
 */
 
 /**
  * liblwgeom versions
  */
-#define LIBLWGEOM_VERSION "2.2.1"
+#define LIBLWGEOM_VERSION "2.4.1"
 #define LIBLWGEOM_VERSION_MAJOR "2"
-#define LIBLWGEOM_VERSION_MINOR "2"
-#define LIBLWGEOM_GEOS_VERSION "35"
+#define LIBLWGEOM_VERSION_MINOR "4"
+#define LIBLWGEOM_GEOS_VERSION "36"
 
 /** Return lwgeom version string (not to be freed) */
 const char* lwgeom_version(void);
@@ -104,7 +118,7 @@ typedef enum LWORD_T {
 
 /**********************************************************************
 ** Spherical radius.
-** Moritz, H. (1980). Geodetic Reference System 1980, by resolution of 
+** Moritz, H. (1980). Geodetic Reference System 1980, by resolution of
 ** the XVII General Assembly of the IUGG in Canberra.
 ** http://en.wikipedia.org/wiki/Earth_radius
 ** http://en.wikipedia.org/wiki/World_Geodetic_System
@@ -117,9 +131,10 @@ typedef enum LWORD_T {
 
 
 /**
-* Macros for manipulating the 'flags' byte. A uint8_t used as follows: 
-* ---RGBMZ
-* Three unused bits, followed by ReadOnly, Geodetic, HasBBox, HasM and HasZ flags.
+* Macros for manipulating the 'flags' byte. A uint8_t used as follows:
+* VVSRGBMZ
+* Version bit, followed by
+* Validty, Solid, ReadOnly, Geodetic, HasBBox, HasM and HasZ flags.
 */
 #define FLAGS_GET_Z(flags) ((flags) & 0x01)
 #define FLAGS_GET_M(flags) (((flags) & 0x02)>>1)
@@ -146,7 +161,7 @@ typedef enum LWORD_T {
 * ZM Flags = Bottom 2 bits.
 */
 
-#define TYPMOD_GET_SRID(typmod) ((((typmod) & 0x1FFFFF00)<<3)>>11)
+#define TYPMOD_GET_SRID(typmod) ((((typmod) & 0x0FFFFF00) - ((typmod) & 0x10000000)) >> 8)
 #define TYPMOD_SET_SRID(typmod, srid) ((typmod) = (((typmod) & 0xE00000FF) | ((srid & 0x001FFFFF)<<8)))
 #define TYPMOD_GET_TYPE(typmod) ((typmod & 0x000000FC)>>2)
 #define TYPMOD_SET_TYPE(typmod, type) ((typmod) = (typmod & 0xFFFFFF03) | ((type & 0x0000003F)<<2))
@@ -172,8 +187,8 @@ typedef enum LWORD_T {
 #define SRID_UNKNOWN 0
 #define SRID_IS_UNKNOWN(x) ((int)x<=0)
 
-/* 
-** EPSG WGS84 geographics, OGC standard default SRS, better be in 
+/*
+** EPSG WGS84 geographics, OGC standard default SRS, better be in
 ** the SPATIAL_REF_SYS table!
 */
 #define SRID_DEFAULT 4326
@@ -210,7 +225,7 @@ typedef void (*lwdebuglogger)(int level, const char* fmt, va_list ap)
 * @ingroup system
 * @todo take a structure ?
 */
-extern void lwgeom_set_handlers(lwallocator allocator, 
+extern void lwgeom_set_handlers(lwallocator allocator,
         lwreallocator reallocator, lwfreeor freeor, lwreporter errorreporter,
         lwreporter noticereporter);
 
@@ -226,7 +241,7 @@ extern void lwgeom_set_debuglogger(lwdebuglogger debuglogger);
  *
  * The return value from interrupted code is undefined,
  * it is the caller responsibility to not take it in consideration.
- * 
+ *
  */
 extern void lwgeom_request_interrupt(void);
 
@@ -265,8 +280,8 @@ typedef struct
 BOX3D;
 
 /******************************************************************
-* GBOX structure. 
-* We include the flags (information about dimensinality), 
+* GBOX structure.
+* We include the flags (information about dimensinality),
 * so we don't have to constantly pass them
 * into functions that use the GBOX.
 */
@@ -372,7 +387,7 @@ typedef struct
 /******************************************************************
 * LWGEOM (any geometry type)
 *
-* Abstract type, note that 'type', 'bbox' and 'srid' are available in 
+* Abstract type, note that 'type', 'bbox' and 'srid' are available in
 * all geometry variants.
 */
 typedef struct
@@ -624,17 +639,23 @@ extern LWTIN* lwtin_add_lwtriangle(LWTIN *mobj, const LWTRIANGLE *obj);
 extern uint8_t gflags(int hasz, int hasm, int geodetic);
 
 /**
-* Extract the geometry type from the serialized form (it hides in 
+* Extract the geometry type from the serialized form (it hides in
 * the anonymous data area, so this is a handy function).
 */
 extern uint32_t gserialized_get_type(const GSERIALIZED *g);
 
-/** 
-* Returns the size in bytes to read from toast to get the basic 
-* information from a geometry: GSERIALIZED struct, bbox and type 
+/**
+* Returns the size in bytes to read from toast to get the basic
+* information from a geometry: GSERIALIZED struct, bbox and type
 */
 extern uint32_t gserialized_max_header_size(void);
 
+/**
+* Returns the size in bytes of the header, from the start of the
+* object up to the type number.
+*/
+extern uint32_t gserialized_header_size(const GSERIALIZED *gser);
+
 /**
 * Extract the SRID from the serialized form (it is packed into
 * three bytes so this is a handy function).
@@ -650,8 +671,8 @@ extern void gserialized_set_srid(GSERIALIZED *g, int32_t srid);
 /**
 * Check if a #GSERIALIZED is empty without deserializing first.
 * Only checks if the number of elements of the parent geometry
-* is zero, will not catch collections of empty, eg: 
-* GEOMETRYCOLLECTION(POINT EMPTY) 
+* is zero, will not catch collections of empty, eg:
+* GEOMETRYCOLLECTION(POINT EMPTY)
 */
 extern int gserialized_is_empty(const GSERIALIZED *g);
 
@@ -686,6 +707,16 @@ extern int gserialized_get_zm(const GSERIALIZED *gser);
 */
 extern int gserialized_ndims(const GSERIALIZED *gser);
 
+/**
+* Return -1 if g1 is "less than" g2, 1 if g1 is "greater than"
+* g2 and 0 if g1 and g2 are the "same". Equality is evaluated
+* with a memcmp and size check. So it is possible that two
+* identical objects where one lacks a bounding box could be
+* evaluated as non-equal initially. Greater and less than
+* are evaluated by calculating a sortable key from the center
+* point of the object bounds.
+*/
+extern int gserialized_cmp(const GSERIALIZED *g1, const GSERIALIZED *g2);
 
 /**
 * Call this function to drop BBOX and SRID
@@ -696,8 +727,8 @@ extern int gserialized_ndims(const GSERIALIZED *gser);
 extern void lwgeom_drop_bbox(LWGEOM *lwgeom);
 extern void lwgeom_drop_srid(LWGEOM *lwgeom);
 
-/** 
- * Compute a bbox if not already computed 
+/**
+ * Compute a bbox if not already computed
  *
  * After calling this function lwgeom->bbox is only
  * NULL if the geometry is empty.
@@ -708,7 +739,7 @@ extern void lwgeom_add_bbox(LWGEOM *lwgeom);
 */
 extern void lwgeom_add_bbox_deep(LWGEOM *lwgeom, GBOX *gbox);
 
-/** 
+/**
  * Get a non-empty geometry bounding box, computing and
  * caching it if not already there
  *
@@ -717,15 +748,15 @@ extern void lwgeom_add_bbox_deep(LWGEOM *lwgeom, GBOX *gbox);
  */
 extern const GBOX *lwgeom_get_bbox(const LWGEOM *lwgeom);
 
-/** 
-* Determine whether a LWGEOM can contain sub-geometries or not 
+/**
+* Determine whether a LWGEOM can contain sub-geometries or not
 */
 extern int lwgeom_is_collection(const LWGEOM *lwgeom);
 
 /******************************************************************/
 /* Functions that work on type numbers */
 
-/** 
+/**
 * Determine whether a type number is a collection or not
 */
 extern int lwtype_is_collection(uint8_t type);
@@ -736,7 +767,7 @@ extern int lwtype_is_collection(uint8_t type);
 extern int lwtype_get_collectiontype(uint8_t type);
 
 /**
-* Return the type name string associated with a type number 
+* Return the type name string associated with a type number
 * (e.g. Point, LineString, Polygon)
 */
 extern const char *lwtype_name(uint8_t type);
@@ -792,25 +823,25 @@ extern POINT2D getPoint2d(const POINTARRAY *pa, int n);
 extern int getPoint2d_p(const POINTARRAY *pa, int n, POINT2D *point);
 
 /**
-* Returns a POINT2D pointer into the POINTARRAY serialized_ptlist, 
+* Returns a POINT2D pointer into the POINTARRAY serialized_ptlist,
 * suitable for reading from. This is very high performance
-* and declared const because you aren't allowed to muck with the 
+* and declared const because you aren't allowed to muck with the
 * values, only read them.
 */
 extern const POINT2D* getPoint2d_cp(const POINTARRAY *pa, int n);
 
 /**
-* Returns a POINT3DZ pointer into the POINTARRAY serialized_ptlist, 
+* Returns a POINT3DZ pointer into the POINTARRAY serialized_ptlist,
 * suitable for reading from. This is very high performance
-* and declared const because you aren't allowed to muck with the 
+* and declared const because you aren't allowed to muck with the
 * values, only read them.
 */
 extern const POINT3DZ* getPoint3dz_cp(const POINTARRAY *pa, int n);
 
 /**
-* Returns a POINT4D pointer into the POINTARRAY serialized_ptlist, 
+* Returns a POINT4D pointer into the POINTARRAY serialized_ptlist,
 * suitable for reading from. This is very high performance
-* and declared const because you aren't allowed to muck with the 
+* and declared const because you aren't allowed to muck with the
 * values, only read them.
 */
 extern const POINT4D* getPoint4d_cp(const POINTARRAY *pa, int n);
@@ -821,7 +852,7 @@ extern const POINT4D* getPoint4d_cp(const POINTARRAY *pa, int n);
  * dimension, the appropriate ordinate values
  * will be extracted from it
  *
- * N must be a valid point index 
+ * N must be a valid point index
  */
 extern void ptarray_set_point4d(POINTARRAY *pa, int n, const POINT4D *p4d);
 
@@ -843,7 +874,7 @@ extern uint8_t *getPoint_internal(const POINTARRAY *pa, int n);
 extern int ptarray_point_size(const POINTARRAY *pa);
 
 
-/** 
+/**
 * Construct an empty pointarray, allocating storage and setting
 * the npoints, but not filling in any information. Should be used in conjunction
 * with ptarray_set_point4d to fill in the information in the array.
@@ -851,12 +882,12 @@ extern int ptarray_point_size(const POINTARRAY *pa);
 extern POINTARRAY* ptarray_construct(char hasz, char hasm, uint32_t npoints);
 
 /**
-* Construct a new #POINTARRAY, <em>copying</em> in the data from ptlist 
+* Construct a new #POINTARRAY, <em>copying</em> in the data from ptlist
 */
 extern POINTARRAY* ptarray_construct_copy_data(char hasz, char hasm, uint32_t npoints, const uint8_t *ptlist);
 
 /**
-* Construct a new #POINTARRAY, <em>referencing</em> to the data from ptlist 
+* Construct a new #POINTARRAY, <em>referencing</em> to the data from ptlist
 */
 extern POINTARRAY* ptarray_construct_reference_data(char hasz, char hasm, uint32_t npoints, uint8_t *ptlist);
 
@@ -868,8 +899,8 @@ extern POINTARRAY* ptarray_construct_reference_data(char hasz, char hasm, uint32
 extern POINTARRAY* ptarray_construct_empty(char hasz, char hasm, uint32_t maxpoints);
 
 /**
-* Append a point to the end of an existing #POINTARRAY 
-* If allow_duplicate is LW_TRUE, then a duplicate point will
+* Append a point to the end of an existing #POINTARRAY
+* If allow_duplicate is LW_FALSE, then a duplicate point will
 * not be added.
 */
 extern int ptarray_append_point(POINTARRAY *pa, const POINT4D *pt, int allow_duplicates);
@@ -956,7 +987,7 @@ extern LWGEOM* lwgeom_force_4d(const LWGEOM *geom);
 extern LWGEOM* lwgeom_simplify(const LWGEOM *igeom, double dist, int preserve_collapsed);
 extern LWGEOM* lwgeom_set_effective_area(const LWGEOM *igeom, int set_area, double area);
 
-/* 
+/*
  * Force to use SFS 1.1 geometry type
  * (rather than SFS 1.2 and/or SQL/MM)
  */
@@ -1009,8 +1040,8 @@ extern int lwcurvepoly_add_ring(LWCURVEPOLY *poly, LWGEOM *ring);
 extern int lwcompound_add_lwgeom(LWCOMPOUND *comp, LWGEOM *geom);
 
 /**
-* Construct an equivalent compound curve from a linestring. 
-* Compound curves can have linear components, so this works fine 
+* Construct an equivalent compound curve from a linestring.
+* Compound curves can have linear components, so this works fine
 */
 extern LWCOMPOUND* lwcompound_construct_from_lwline(const LWLINE *lwpoly);
 
@@ -1027,7 +1058,12 @@ extern LWCURVEPOLY* lwcurvepoly_construct_from_lwpoly(LWPOLY *lwpoly);
 
 extern int lwcollection_ngeoms(const LWCOLLECTION *col);
 
-/* Given a generic geometry/collection, return the "simplest" form. */
+/* Given a generic geometry/collection, return the "simplest" form.
+ * The elements of the homogenized collection are references to the
+ * input geometry; a deep clone is not performed.
+ * TODO: consider returning a geometry that does not reference the
+ * input
+ * */
 extern LWGEOM *lwgeom_homogenize(const LWGEOM *geom);
 
 
@@ -1036,6 +1072,9 @@ extern LWGEOM *lwgeom_homogenize(const LWGEOM *geom);
  ******************************************************************/
 
 LWGEOM *lwcollection_getsubgeom(LWCOLLECTION *col, int gnum);
+
+/* WARNING: the output will contain references to geometries in the input, */
+/* so the result must be carefully released, not freed. */
 LWCOLLECTION* lwcollection_extract(LWCOLLECTION *col, int type);
 
 
@@ -1070,7 +1109,7 @@ void expand_box3d(BOX3D *box, double d);
 
 /*
 * The *_free family of functions frees *all* memory associated
-* with the pointer. When the recursion gets to the level of the 
+* with the pointer. When the recursion gets to the level of the
 * POINTARRAY, the POINTARRAY is only freed if it is not flagged
 * as "read only". LWGEOMs constructed on top of GSERIALIZED
 * from PgSQL use read only point arrays.
@@ -1126,8 +1165,6 @@ extern void printLWTIN(LWTIN *tin);
 
 extern float  next_float_down(double d);
 extern float  next_float_up(double d);
-extern double next_double_down(float d);
-extern double next_double_up(float d);
 
 /* general utilities 2D */
 extern double  distance2d_pt_pt(const POINT2D *p1, const POINT2D *p2);
@@ -1191,18 +1228,35 @@ extern char* lwpoint_to_latlon(const LWPOINT *p, const char *format);
 extern int lwgeom_startpoint(const LWGEOM* lwgeom, POINT4D* pt);
 
 /**
-* Ensure the outer ring is clockwise oriented and all inner rings 
+* Ensure the outer ring is clockwise oriented and all inner rings
 * are counter-clockwise.
 */
 extern void lwgeom_force_clockwise(LWGEOM *lwgeom);
 extern void lwpoly_force_clockwise(LWPOLY *poly);
 extern void lwtriangle_force_clockwise(LWTRIANGLE *triangle);
 
+extern int lwgeom_is_clockwise(LWGEOM *lwgeom);
+extern int lwpoly_is_clockwise(LWPOLY *poly);
+extern int lwtriangle_is_clockwise(LWTRIANGLE *triangle);
 
 extern void interpolate_point4d(POINT4D *A, POINT4D *B, POINT4D *I, double F);
 
 void lwgeom_longitude_shift(LWGEOM *lwgeom);
 
+/**
+ * @brief wrap geometry on given cut x value
+ *
+ * For a positive amount, shifts anything that is on the left
+ * of "cutx" to the right by the given amount.
+ *
+ * For a negative amount, shifts anything that is on the right
+ * of "cutx" to the left by the given absolute amount.
+ *
+ * @param cutx the X value to perform wrapping on
+ * @param amount shift amount and wrapping direction
+ */
+LWGEOM *lwgeom_wrapx(const LWGEOM *lwgeom, double cutx, double amount);
+
 
 /**
 * @brief Check whether or not a lwgeom is big enough to warrant a bounding box.
@@ -1265,7 +1319,7 @@ char lwcircstring_same(const LWCIRCSTRING *p1, const LWCIRCSTRING *p2);
  *
  * #GBOX are copied
  *
- * @see ptarray_clone 
+ * @see ptarray_clone
  */
 extern LWGEOM *lwgeom_clone(const LWGEOM *lwgeom);
 
@@ -1291,7 +1345,7 @@ extern LWCIRCSTRING* lwcircstring_construct(int srid, GBOX *bbox, POINTARRAY *po
 extern LWPOLY* lwpoly_construct(int srid, GBOX *bbox, uint32_t nrings, POINTARRAY **points);
 extern LWCURVEPOLY* lwcurvepoly_construct(int srid, GBOX *bbox, uint32_t nrings, LWGEOM **geoms);
 extern LWTRIANGLE* lwtriangle_construct(int srid, GBOX *bbox, POINTARRAY *points);
-extern LWCOLLECTION* lwcollection_construct(uint8_t type, int srid, GBOX *bbox, uint32_t ngeoms, LWGEOM **geoms); 
+extern LWCOLLECTION* lwcollection_construct(uint8_t type, int srid, GBOX *bbox, uint32_t ngeoms, LWGEOM **geoms);
 /*
 * Empty geometry constructors.
 */
@@ -1322,8 +1376,11 @@ extern LWLINE *lwline_addpoint(LWLINE *line, LWPOINT *point, uint32_t where);
 extern LWLINE *lwline_removepoint(LWLINE *line, uint32_t which);
 extern void lwline_setPoint4d(LWLINE *line, uint32_t which, POINT4D *newpoint);
 extern LWPOLY *lwpoly_from_lwlines(const LWLINE *shell, uint32_t nholes, const LWLINE **holes);
+extern LWPOLY *lwpoly_construct_rectangle(char hasz, char hasm, POINT4D *p1, POINT4D *p2, POINT4D *p3, POINT4D *p4);
+extern LWPOLY *lwpoly_construct_envelope(int srid, double x1, double y1, double x2, double y2);
+extern LWPOLY *lwpoly_construct_circle(int srid, double x, double y, double radius, uint32_t segments_per_quarter, char exterior);
 extern LWTRIANGLE *lwtriangle_from_lwline(const LWLINE *shell);
-
+extern LWMPOINT *lwmpoint_from_lwgeom(const LWGEOM *g); /* Extract the coordinates of an LWGEOM into an LWMPOINT */
 
 /* Some point accessors */
 extern double lwpoint_get_x(const LWPOINT *point);
@@ -1366,14 +1423,14 @@ extern int lwgeom_ndims(const LWGEOM *geom);
 extern double ptarray_locate_point(const POINTARRAY *pa, const POINT4D *pt, double *dist, POINT4D *p_located);
 
 /**
-* Add a measure dimension to a line, interpolating linearly from the start 
+* Add a measure dimension to a line, interpolating linearly from the start
 * to the end value.
 */
 extern LWLINE *lwline_measured_from_lwline(const LWLINE *lwline, double m_start, double m_end);
 extern LWMLINE* lwmline_measured_from_lwmline(const LWMLINE *lwmline, double m_start, double m_end);
 
 /**
-* Determine the location(s) along a measured line where m occurs and 
+* Determine the location(s) along a measured line where m occurs and
 * return as a multipoint. Offset to left (positive) or right (negative).
 */
 extern LWGEOM* lwgeom_locate_along(const LWGEOM *lwin, double m, double offset);
@@ -1426,6 +1483,19 @@ extern LWLINE *lwline_segmentize2d(LWLINE *line, double dist);
 extern LWPOLY *lwpoly_segmentize2d(LWPOLY *line, double dist);
 extern LWCOLLECTION *lwcollection_segmentize2d(LWCOLLECTION *coll, double dist);
 
+/*
+ * Point density functions
+ */
+extern LWMPOINT *lwpoly_to_points(const LWPOLY *poly, int npoints);
+extern LWMPOINT *lwmpoly_to_points(const LWMPOLY *mpoly, int npoints);
+extern LWMPOINT *lwgeom_to_points(const LWGEOM *lwgeom, int npoints);
+
+/*
+ * Geometric median
+ */
+extern LWPOINT* lwgeom_median(const LWGEOM *g, double tol, uint32_t maxiter, char fail_if_not_converged);
+extern LWPOINT* lwmpoint_median(const LWMPOINT *g, double tol, uint32_t maxiter, char fail_if_not_converged);
+
 /**
 * Calculate the GeoHash (http://geohash.org) string for a geometry. Caller must free.
 */
@@ -1457,7 +1527,7 @@ int lwline_crossing_direction(const LWLINE *l1, const LWLINE *l2);
 LWCOLLECTION* lwgeom_clip_to_ordinate_range(const LWGEOM *lwin, char ordinate, double from, double to, double offset);
 
 /**
- * Macros for specifying GML options. 
+ * Macros for specifying GML options.
  * @{
  */
 /** For GML3 only, include srsDimension attribute in output */
@@ -1475,7 +1545,7 @@ LWCOLLECTION* lwgeom_clip_to_ordinate_range(const LWGEOM *lwin, char ordinate, d
 /** @} */
 
 /**
- * Macros for specifying X3D options. 
+ * Macros for specifying X3D options.
  * @{
  */
 /** For flip X/Y coordinates to Y/X */
@@ -1522,7 +1592,7 @@ extern LWGEOM* lwgeom_from_encoded_polyline(const char *encodedpolyline, int pre
 extern void spheroid_init(SPHEROID *s, double a, double b);
 
 /**
-* Calculate the geodetic distance from lwgeom1 to lwgeom2 on the spheroid. 
+* Calculate the geodetic distance from lwgeom1 to lwgeom2 on the spheroid.
 * A spheroid with major axis == minor axis will be treated as a sphere.
 * Pass in a tolerance in spheroid units.
 */
@@ -1534,7 +1604,7 @@ extern double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwge
 extern LWPOINT* lwgeom_project_spheroid(const LWPOINT *r, const SPHEROID *spheroid, double distance, double azimuth);
 
 /**
-* Derive a new geometry with vertices added to ensure no vertex is more 
+* Derive a new geometry with vertices added to ensure no vertex is more
 * than max_seg_length (in radians) from any other vertex.
 */
 extern LWGEOM* lwgeom_segmentize_sphere(const LWGEOM *lwg_in, double max_seg_length);
@@ -1568,10 +1638,31 @@ extern double lwgeom_length_spheroid(const LWGEOM *geom, const SPHEROID *s);
 */
 extern int lwgeom_covers_lwgeom_sphere(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2);
 
+typedef struct {
+	POINT2D* center;
+	double radius;
+} LWBOUNDINGCIRCLE;
+
+extern void lwboundingcircle_destroy(LWBOUNDINGCIRCLE* c);
+
+/* Calculates the minimum circle that encloses all of the points in g, using a
+ * two-dimensional implementation of the algorithm proposed in:
+ *
+ * Welzl, Emo (1991), "Smallest enclosing disks (balls and elipsoids)."
+ * New Results and Trends in Computer Science (H. Maurer, Ed.), Lecture Notes
+ * in Computer Science, 555 (1991) 359-370.
+ *
+ * Available online at the time of this writing at
+ * https://www.inf.ethz.ch/personal/emo/PublFiles/SmallEnclDisk_LNCS555_91.pdf
+ *
+ * Returns NULL if the circle could not be calculated.
+ */
+extern LWBOUNDINGCIRCLE* lwgeom_calculate_mbc(const LWGEOM* g);
+
 /**
 * Remove repeated points!
 */
-extern LWGEOM* lwgeom_remove_repeated_points(LWGEOM *in, double tolerance);
+extern LWGEOM* lwgeom_remove_repeated_points(const LWGEOM *in, double tolerance);
 
 extern char lwtriangle_is_repeated_points(LWTRIANGLE *triangle);
 
@@ -1598,13 +1689,59 @@ extern void lwgeom_swap_ordinates(LWGEOM *in, LWORD o1, LWORD o2);
 */
 extern LWGEOM* lwgeom_flip_coordinates(LWGEOM *in);
 
+struct LWPOINTITERATOR;
+typedef struct LWPOINTITERATOR LWPOINTITERATOR;
+
+/**
+ * Create a new LWPOINTITERATOR over supplied LWGEOM*
+ */
+extern LWPOINTITERATOR* lwpointiterator_create(const LWGEOM* g);
+
+/**
+ * Create a new LWPOINTITERATOR over supplied LWGEOM*
+ * Supports modification of coordinates during iteration.
+ */
+extern LWPOINTITERATOR* lwpointiterator_create_rw(LWGEOM* g);
+
+/**
+ * Free all memory associated with the iterator
+ */
+extern void lwpointiterator_destroy(LWPOINTITERATOR* s);
+
+/**
+ * Returns LW_TRUE if there is another point available in the iterator.
+ */
+extern int lwpointiterator_has_next(LWPOINTITERATOR* s);
+
+/**
+ * Attempts to replace the next point int the iterator with p, and advances
+ * the iterator to the next point.
+ * Returns LW_SUCCESS if the assignment was successful, LW_FAILURE otherwise.
+ * */
+extern int lwpointiterator_modify_next(LWPOINTITERATOR* s, const POINT4D* p);
+
+/**
+ * Attempts to assign the next point in the iterator to p, and advances
+ * the iterator to the next point.  If p is NULL, the iterator will be
+ * advanced without reading a point.
+ * Returns LW_SUCCESS if the assignment was successful, LW_FAILURE otherwise.
+ * */
+extern int lwpointiterator_next(LWPOINTITERATOR* s, POINT4D* p);
+
+/**
+ * Attempts to assigns the next point in the iterator to p.  Does not advance.
+ * Returns LW_SUCCESS if the assignment was successful, LW_FAILURE otherwise.
+ */
+extern int lwpointiterator_peek(LWPOINTITERATOR* s, POINT4D* p);
+
+
 /**
 * Convert a single hex digit into the corresponding char
 */
 extern uint8_t parse_hex(char *str);
 
 /**
-* Convert a char into a human readable hex digit 
+* Convert a char into a human readable hex digit
 */
 extern void deparse_hex(uint8_t str, char *result);
 
@@ -1629,14 +1766,14 @@ extern GBOX* gserialized_calculate_gbox_geocentric(const GSERIALIZED *g);
 int gserialized_calculate_gbox_geocentric_p(const GSERIALIZED *g, GBOX *g_box);
 
 /**
-* Return a WKT representation of the gserialized geometry. 
+* Return a WKT representation of the gserialized geometry.
 * Caller is responsible for disposing of the char*.
 */
 extern char* gserialized_to_string(const GSERIALIZED *g);
 
 /**
-* Return a copy of the input serialized geometry. 
-*/ 
+* Return a copy of the input serialized geometry.
+*/
 extern GSERIALIZED* gserialized_copy(const GSERIALIZED *g);
 
 /**
@@ -1660,7 +1797,7 @@ extern int lwgeom_force_geodetic(LWGEOM *geom);
 extern void lwgeom_set_geodetic(LWGEOM *geom, int value);
 
 /**
-* Calculate the geodetic bounding box for an LWGEOM. Z/M coordinates are 
+* Calculate the geodetic bounding box for an LWGEOM. Z/M coordinates are
 * ignored for this calculation. Pass in non-null, geodetic bounding box for function
 * to fill out. LWGEOM must have been built from a GSERIALIZED to provide
 * double aligned point arrays.
@@ -1668,7 +1805,7 @@ extern void lwgeom_set_geodetic(LWGEOM *geom, int value);
 extern int lwgeom_calculate_gbox_geodetic(const LWGEOM *geom, GBOX *gbox);
 
 /**
-* Calculate the 2-4D bounding box of a geometry. Z/M coordinates are honored 
+* Calculate the 2-4D bounding box of a geometry. Z/M coordinates are honored
 * for this calculation, though for curves they are not included in calculations
 * of curvature.
 */
@@ -1728,6 +1865,11 @@ extern int gbox_union(const GBOX *g1, const GBOX *g2, GBOX *gout);
 */
 extern void gbox_expand(GBOX *g, double d);
 
+/**
+* Move the box minimums down and the maximums up by the distances provided.
+*/
+extern void gbox_expand_xyzm(GBOX *g, double dx, double dy, double dz, double dm);
+
 /**
 * Initialize a #GBOX using the values of the point.
 */
@@ -1759,17 +1901,17 @@ extern GBOX* gbox_copy(const GBOX *gbox);
 extern GBOX* gbox_from_string(const char *str);
 
 /**
-* Return #LW_TRUE if the #GBOX overlaps, #LW_FALSE otherwise. 
+* Return #LW_TRUE if the #GBOX overlaps, #LW_FALSE otherwise.
 */
 extern int gbox_overlaps(const GBOX *g1, const GBOX *g2);
 
 /**
-* Return #LW_TRUE if the #GBOX overlaps on the 2d plane, #LW_FALSE otherwise. 
+* Return #LW_TRUE if the #GBOX overlaps on the 2d plane, #LW_FALSE otherwise.
 */
 extern int gbox_overlaps_2d(const GBOX *g1, const GBOX *g2);
 
 /**
-* Return #LW_TRUE if the first #GBOX contains the second on the 2d plane, #LW_FALSE otherwise. 
+* Return #LW_TRUE if the first #GBOX contains the second on the 2d plane, #LW_FALSE otherwise.
 */
 extern int  gbox_contains_2d(const GBOX *g1, const GBOX *g2);
 
@@ -1779,7 +1921,7 @@ extern int  gbox_contains_2d(const GBOX *g1, const GBOX *g2);
 extern void gbox_duplicate(const GBOX *original, GBOX *duplicate);
 
 /**
-* Return the number of bytes necessary to hold a #GBOX of this dimension in 
+* Return the number of bytes necessary to hold a #GBOX of this dimension in
 * serialized form.
 */
 extern size_t gbox_serialized_size(uint8_t flags);
@@ -1814,8 +1956,14 @@ extern void gbox_float_round(GBOX *gbox);
 extern int gbox_is_valid(const GBOX *gbox);
 
 /**
-* Utility function to get type number from string. For example, a string 'POINTZ' 
-* would return type of 1 and z of 1 and m of 0. Valid 
+* Return a sortable key based on the center point of the
+* GBOX.
+*/
+extern uint64_t gbox_get_sortable_hash(const GBOX *g);
+
+/**
+* Utility function to get type number from string. For example, a string 'POINTZ'
+* would return type of 1 and z of 1 and m of 0. Valid
 */
 extern int geometry_type_from_string(const char *str, uint8_t *type, int *z, int *m);
 
@@ -1830,10 +1978,10 @@ extern size_t gserialized_from_lwgeom_size(const LWGEOM *geom);
 * Allocate a new #GSERIALIZED from an #LWGEOM. For all non-point types, a bounding
 * box will be calculated and embedded in the serialization. The geodetic flag is used
 * to control the box calculation (cartesian or geocentric). If set, the size pointer
-* will contain the size of the final output, which is useful for setting the PgSQL 
+* will contain the size of the final output, which is useful for setting the PgSQL
 * VARSIZE information.
 */
-extern GSERIALIZED* gserialized_from_lwgeom(LWGEOM *geom, int is_geodetic, size_t *size);
+extern GSERIALIZED* gserialized_from_lwgeom(LWGEOM *geom, size_t *size);
 
 /**
 * Allocate a new #LWGEOM from a #GSERIALIZED. The resulting #LWGEOM will have coordinates
@@ -1851,7 +1999,7 @@ extern int gserialized_get_gbox_p(const GSERIALIZED *g, GBOX *gbox);
 
 /**
  * Parser check flags
- * 
+ *
  *  @see lwgeom_from_wkb
  *  @see lwgeom_from_hexwkb
  *  @see lwgeom_parse_wkt
@@ -2033,13 +2181,80 @@ extern uint8_t* lwgeom_to_twkb(const LWGEOM *geom, uint8_t variant, int8_t preci
 extern uint8_t* lwgeom_to_twkb_with_idlist(const LWGEOM *geom, int64_t *idlist, uint8_t variant, int8_t precision_xy, int8_t precision_z, int8_t precision_m, size_t *twkb_size);
 
 /*******************************************************************************
- * SQLMM internal functions - TODO: Move into separate header files
+ * SQLMM internal functions
  ******************************************************************************/
 
 int lwgeom_has_arc(const LWGEOM *geom);
 LWGEOM *lwgeom_stroke(const LWGEOM *geom, uint32_t perQuad);
 LWGEOM *lwgeom_unstroke(const LWGEOM *geom);
 
+/**
+ * Semantic of the `tolerance` argument passed to
+ * lwcurve_linearize
+ */
+typedef enum {
+	/**
+	 * Tolerance expresses the number of segments to use
+	 * for each quarter of circle (quadrant). Must be
+	 * an integer.
+	 */
+	LW_LINEARIZE_TOLERANCE_TYPE_SEGS_PER_QUAD = 0,
+	/**
+	 * Tolerance expresses the maximum distance between
+	 * an arbitrary point on the curve and the closest
+	 * point to it on the resulting approximation, in
+	 * cartesian units.
+	 */
+	LW_LINEARIZE_TOLERANCE_TYPE_MAX_DEVIATION = 1,
+	/**
+	 * Tolerance expresses the maximum angle between
+	 * the radii generating approximation line vertices,
+	 * given in radiuses. A value of 1 would result
+	 * in an approximation of a semicircle composed by
+	 * 180 segments
+	 */
+	LW_LINEARIZE_TOLERANCE_TYPE_MAX_ANGLE = 2
+} LW_LINEARIZE_TOLERANCE_TYPE;
+
+typedef enum {
+  /**
+   * Symmetric linearization means that the output
+   * vertices would be the same no matter the order
+   * of the points defining the input curve.
+   */
+	LW_LINEARIZE_FLAG_SYMMETRIC = 1 << 0,
+
+  /**
+   * Retain angle instructs the engine to try its best
+   * to retain the requested angle between generating
+   * radii (where angle can be given explicitly with
+   * LW_LINEARIZE_TOLERANCE_TYPE_MAX_ANGLE or implicitly
+   * with LW_LINEARIZE_TOLERANCE_TYPE_SEGS_PER_QUAD or
+   * LW_LINEARIZE_TOLERANCE_TYPE_MAX_DEVIATION).
+   *
+   * It only makes sense with LW_LINEARIZE_FLAG_SYMMETRIC
+   * which would otherwise reduce the angle as needed to
+   * keep it constant among all radiis so that all
+   * segments are of the same length.
+   *
+   * When this flag is set, the first and last generating
+   * angles (and thus the first and last segments) may
+   * instead be smaller (shorter) than the others.
+   *
+   */
+	LW_LINEARIZE_FLAG_RETAIN_ANGLE = 1 << 1
+} LW_LINEARIZE_FLAGS;
+
+/**
+ * @param geom input geometry
+ * @param tol tolerance, semantic driven by tolerance_type
+ * @param tolerance_type see LW_LINEARIZE_TOLERANCE_TYPE
+ * @param flags bitwise OR of operational flags, see LW_LINEARIZE_FLAGS
+ *
+ * @return a newly allocated LWGEOM
+ */
+extern LWGEOM* lwcurve_linearize(const LWGEOM *geom, double tol, LW_LINEARIZE_TOLERANCE_TYPE type, int flags);
+
 /*******************************************************************************
  * GEOS proxy functions on LWGEOM
  ******************************************************************************/
@@ -2054,6 +2269,7 @@ LWGEOM *lwgeom_normalize(const LWGEOM *geom);
 LWGEOM *lwgeom_intersection(const LWGEOM *geom1, const LWGEOM *geom2);
 LWGEOM *lwgeom_difference(const LWGEOM *geom1, const LWGEOM *geom2);
 LWGEOM *lwgeom_symdifference(const LWGEOM* geom1, const LWGEOM* geom2);
+LWGEOM *lwgeom_centroid(const LWGEOM* geom);
 LWGEOM *lwgeom_union(const LWGEOM *geom1, const LWGEOM *geom2);
 LWGEOM *lwgeom_linemerge(const LWGEOM *geom1);
 LWGEOM *lwgeom_unaryunion(const LWGEOM *geom1);
@@ -2066,7 +2282,7 @@ LWCOLLECTION *lwgeom_subdivide(const LWGEOM *geom, int maxvertices);
  * @param geom1 the geometry to snap
  * @param geom2 the geometry to snap to
  * @param tolerance the distance under which vertices and segments are snapped
- * 
+ *
  * Requires GEOS-3.3.0+
  */
 LWGEOM* lwgeom_snap(const LWGEOM* geom1, const LWGEOM* geom2, double tolerance);
@@ -2171,11 +2387,35 @@ LWGEOM* lwgeom_node(const LWGEOM* lwgeom_in);
  * triangulation on them.
  *
  * @param geom the input geometry
- * @param tolerance an optional snapping tolerance for improved tolerance
+ * @param tolerance an optional snapping tolerance for improved robustness
  * @param edgeOnly if non-zero the result will be a MULTILINESTRING,
  *                 otherwise it'll be a COLLECTION of polygons.
  */
 LWGEOM* lwgeom_delaunay_triangulation(const LWGEOM *geom, double tolerance, int edgeOnly);
 
+/**
+ * Take vertices of a geometry and build the Voronoi diagram
+ *
+ * @param g the input geometry
+ * @param env an optional envelope for clipping the results
+ * @param tolerance an optional snapping tolerance for improved robustness
+ * @param output_edges if non-zero the result will be a MULTILINESTRING,
+ *                 otherwise it'll be a COLLECTION of polygons.
+ *
+ * Requires GEOS-3.5.0 or higher
+ */
+LWGEOM* lwgeom_voronoi_diagram(const LWGEOM* g, const GBOX* env, double tolerance, int output_edges);
+
+/**
+* Take a list of LWGEOMs and a number of clusters and return an integer
+* array indicating which cluster each geometry is in.
+*
+* @param geoms the input array of LWGEOM pointers
+* @param ngeoms the number of elements in the array
+* @param k the number of clusters to calculate
+*/
+int * lwgeom_cluster_2d_kmeans(const LWGEOM **geoms, int ngeoms, int k);
+
+
 #endif /* !defined _LIBLWGEOM_H  */
 
